/*-------------------------------------------------------------------
Name......... audiohsfdrv.h
Author....... Ed Langley
Date......... 28/07/2005
Description.. This file contains the definitions and data types for the
............. Pluto 6 Heber Sound Format audio driver.
-------------------------------------------------------------------*/

/*-------------------------------------------------------------------
DECLARATION FORMATS
Type defs........... UPPER CASE
#defines............ UPPER CASE
Global Functions.... GlobalFunction();
Static Functions.... staticfunction();
Global Variables.... GlobalVariable;
Static Variables.... staticvariable;
Local Variables..... localvariable;
Structure Elements.. Structure.element;
-------------------------------------------------------------------*/

/****	Defines */
#ifndef INTERFACE_AUDIOHSF
#define INTERFACE_AUDIOHSF

/* events */
#define	HSF_10MS_ISR				0
#define HSF_DMA_ISR					1

#define	NUM_CHANNELS				4

/* error codes - these have been given the same values as the
   equivalents in the other driver, in case references to the other
   defines are not replaced in game code */
#define HSF_SUCCESS					0
#define HSFERR_FILENOTFOUND			1
#define	HSFERR_CHANNELBUSY			1
#define	HSFERR_INVALIDCHANNEL		2


/* buffer size to contain >45ms of audio samples @ 8bits per sample
	compressed, 32khz sample rate rounded up to nearest sector */
#define FILE_BUF_SIZE	1536 /* bytes */

#define FILE_LENGTH_FIELD_SIZE_BYTES	4
#define FILE_LENGTH_FIELD_SIZE_WORDS	2

/* sectors to load per channel */
#define SECTORS_TO_LOAD	3

/* buffer size to contain 2 channels of audio data after
	decompression (above *2 *2 then /2 because bytes->words */
#define DMA_BUF_SIZE	3072 /* words */
#define	DMA_BUF_SIZE_BYTES	6144 /* bytes */

/* masks for audio volume register handling */
#define LOWER_NIBBLE_MASK			0x0f
#define UPPER_NIBBLE_MASK			0xf0

/****	Type Definitions */

/* driver infrastructure */
typedef	struct audiohsfcfg
{
	const	DEVICE*	storagedevice;
} AUDIOHSFCFG;

typedef	struct	audiohsfvars
{
	const	AUDIOHSFCFG*	config;
} AUDIOHSFVARS;

/* state enumerations */
typedef	enum
{
	CHANNEL_INACTIVE,
	CHANNEL_STARTING,
	CHANNEL_ACTIVE
} AUDIO_STATUS;

typedef	enum
{
	HSF_CHAN_LEFT0,
	HSF_CHAN_RIGHT0,
	HSF_CHAN_LEFT1,
	HSF_CHAN_RIGHT1
} CURRENT_CHANNEL_STATE;

typedef	enum
{
	EMPTY,
	FILLING,
	FULL,
	ACTIVE
} DMA_BUFFER_STATE;


/* channel data */
typedef struct hsfchannelinfo
{
	FILE_DESCRIPTOR *file;	/* File pointer */
	ULONG	file_length;	/* File length in sectors */
	ULONG	currentpos;		/* Current file position in sectors */
	UBYTE	looped;			/* 0 if not looped, 1 if looped.*/
	UBYTE	dualmono;		/* play mono data on both left and right
							   channels, set in LEFT0/1 */
	AUDIO_STATUS	status;	/* current state of the channel */
	UBYTE	active_mask;	/* bit mask - ANDed with a global var to speed
							   up checking for active channels */

	/* buffer containing compressed ULAW data: */
	union
	{
		UBYTE	bytes[FILE_BUF_SIZE];
		UWORD	words[FILE_BUF_SIZE/2];
		ULONG	longs[FILE_BUF_SIZE/4];
	} fileBuffer;
} HSFCHANNELINFO;

typedef struct statebuffer
{
	volatile UWORD bufix;
	volatile UWORD currentstate;
	volatile UWORD event;
} STATEBUFFER;


/****** Prototypes ******/

#include "audiohsfdrv.p"

#endif

/*************************************************************************/

