/*-------------------------------------------------------------------
Name......... bill.c
Author....... Jon Carroll
Date......... 23/04/07
Description.. Header file for generic bill driver.
-------------------------------------------------------------------*/

/*-------------------------------------------------------------------
DECLARATION FORMATS
Type defs........... UPPER CASE
#defines............ UPPER CASE
Global Functions.... GlobalFunction();
Static Functions.... staticfunction();
Global Variables.... GlobalVariable;
Static Variables.... staticvariable;
Local Variables..... localvariable;
Structure Elements.. Structure.element;
-------------------------------------------------------------------*/
#include	"hardware.h"
#include	"peripherals.h"
#include	"interface.h"
#include	"bill.h"
#include	"bill.sp"

/*-------------------------------------------------------------------
Name......... EnableBillProcessing()
Author....... Jon Carroll
Date......... 24/04/2007
Description..
Parameters... 1) Address of the device config
Return....... None
History...... 24/04/07 - Initial version - JC
--------------------------------------------------------------------*/

void EnableBillProcessing(const DEVICE* dev)
{
	const BILLSERVICES* bser;

	bser=dev->services;
	bser->enable_bill_processing(dev);
}



/*-------------------------------------------------------------------
Name......... DisableBillProcessing()
Author....... Jon Carroll
Date......... 24/04/2007
Description..
Parameters... 1) Address of the device config
Return....... None
History...... 24/04/07 - Initial version - JC
--------------------------------------------------------------------*/

void DisableBillProcessing(const DEVICE* dev)
{
	const BILLSERVICES* bser;

	bser=dev->services;
	bser->disable_bill_processing(dev);
}


/*-------------------------------------------------------------------
Name......... EnableAllBills()
Author....... Jon Carroll
Date......... 28/03/2007
Description.. Enable all bills
Parameters... None
Return....... None
History...... 28/03/07 - Initial version - JC
-------------------------------------------------------------------	*/

void EnableAllBills(const DEVICE* dev)
{
	const BILLSERVICES* bser;

	bser=dev->services;
	bser->enable_all_bills(dev);
}

/*-------------------------------------------------------------------
Name......... EnableBill()
Author....... Jon Carroll
Date......... 23/03/2007
Description.. Processing the bill state
Parameters... None
Return....... None
History...... 23/03/07 - Initial version - JC
-------------------------------------------------------------------	*/

void EnableBill(const DEVICE* dev, UBYTE Bill)
{

	const BILLSERVICES* bser;

	bser=dev->services;
	bser->enable_bill(dev, Bill);
}


/*-------------------------------------------------------------------
Name......... DisableBill()
Author....... Jon Carroll
Date......... 23/03/2007
Description.. Processing the bill state
Parameters... None
Return....... None
History...... 23/03/07 - Initial version - JC
-------------------------------------------------------------------	*/

void DisableBill(const DEVICE* dev, UBYTE Bill)
{
	const BILLSERVICES* bser;

	bser=dev->services;
	bser->disable_bill(dev, Bill);
}


/*-------------------------------------------------------------------
Name......... IsBillInhibited()
Author....... Jon Carroll
Date......... 28/03/2007
Description.. Is the current bill is inhibited
Parameters... None
Return....... Truth
History...... 28/03/07 - Initial version - JC
-------------------------------------------------------------------	*/

BYTE IsBillEnabled(const DEVICE* dev, UBYTE Bill)
{
	const BILLSERVICES* bser;

	bser=dev->services;
	return(bser->is_bill_enabled(dev, Bill));
}

/*-------------------------------------------------------------------
Name......... GetBillErrors()
Author....... Jon Carroll
Date......... 30/05/2007
Description.. Get the last error resported by the device
Parameters... None
Return....... The last error code returned or 0 if no error
History...... 30/05/07 - Initial version - LW
--------------------------------------------------------------------*/

UBYTE GetBillErrors(const DEVICE* dev)
{
	const BILLSERVICES* bser;

	bser=dev->services;

	return(bser->get_bill_errors(dev));
}


/*-------------------------------------------------------------------
Name......... DisableAllBills()
Author....... Jon Carroll
Date......... 28/03/2007
Description.. Disable all bills
Parameters... None
Return....... None
History...... 28/03/07 - Initial version - JC
-------------------------------------------------------------------	*/

void DisableAllBills(const DEVICE* dev)
{
	const BILLSERVICES* bser;

	bser=dev->services;
	bser->disable_all_bills(dev);
}

/*-------------------------------------------------------------------
Name......... EnableBillProcessing()
Author....... Jon Carroll
Date......... 24/04/2007
Description.. Initialise Bill processing
Parameters... 1) Address of the device config
Return....... None
History...... 24/04/07 - Initial version - JC
--------------------------------------------------------------------*/
void SetErrorOnStackerFull(const DEVICE* dev, UBYTE Value)
{
	const BILLSERVICES* bser;

	bser=dev->services;
	bser->set_error_on_stacker_full(dev, Value);
}


/*-------------------------------------------------------------------
Name......... EnableBillProcessing()
Author....... Jon Carroll
Date......... 24/04/2007
Description.. Initialise Bill processing
Parameters... 1) Address of the device config
Return....... None
History...... 24/04/07 - Initial version - JC
--------------------------------------------------------------------*/
UBYTE GetErrorOnStackerFull(const DEVICE* dev)
{
	const BILLSERVICES* bser;

	bser=dev->services;
	return(bser->get_error_on_stacker_full(dev));
}


/*-------------------------------------------------------------------
Name......... EnableBillProcessing()
Author....... Jon Carroll
Date......... 24/04/2007
Description.. Initialise Bill processing
Parameters... 1) Address of the device config
Return....... None
History...... 24/04/07 - Initial version - JC
--------------------------------------------------------------------*/
UBYTE IsStackerRemoved(const DEVICE* dev)
{
	const BILLSERVICES* bser;

	bser=dev->services;
	return(bser->is_stacker_removed(dev));
}


/*-------------------------------------------------------------------
Name......... GetBillCount()
Author....... Jon Carroll
Date......... 23/03/2007
Description.. Processing the bill state
Parameters... 1) Address of the device config
			. 2) The Bill Address
Return....... None
History...... 23/03/07 - Initial version - JC
-------------------------------------------------------------------	*/

UWORD GetBillCount(const DEVICE* dev, UBYTE Bill)
{
	const BILLSERVICES* bser;

	bser=dev->services;
	return(bser->get_bill_count(dev, Bill));

}

/*-------------------------------------------------------------------
Name......... SubtractFromBillCount()
Author....... Jon Carroll
Date......... 23/03/2007
Description.. Processing the bill state
Parameters... 1) Address of the device config
			. 2) The Bill
Return....... None
History...... 23/03/07 - Initial version - JC
-------------------------------------------------------------------	*/

void SubtractFromBillCount(const DEVICE* dev, UBYTE Bill)
{
	const BILLSERVICES* bser;

	bser=dev->services;
	return(bser->subtract_from_bill_counts(dev, Bill));
}


/*-------------------------------------------------------------------
Name......... AddToBillCount()
Author....... Jon Carroll
Date......... 23/03/2007
Description.. Processing the bill state
Parameters... 1) Address of the device config
			. 2) The Bill
Return....... None
History...... 23/03/07 - Initial version - JC
-------------------------------------------------------------------	*/

void AddToBillCount(const DEVICE *dev, UBYTE Bill)
{
	const BILLSERVICES* bser;

	bser=dev->services;
	return(bser->add_to_bill_count(dev, Bill));
}

/*-------------------------------------------------------------------
Name......... AddToBillCount()
Author....... Jon Carroll
Date......... 23/03/2007
Description.. Processing the bill state
Parameters... 1) Address of the device config
			. 2) The polling Rate
Return....... None
History...... 23/03/07 - Initial version - JC
-------------------------------------------------------------------	*/

void SetBillPollingRate(const DEVICE *dev, UWORD Rate)
{
	const BILLSERVICES* bser;

	bser=dev->services;
	return(bser->set_bill_polling_rate(dev, Rate));
}