/*************************************************************************
Title........ fat32.h
Date......... 07 March 2005
Author....... Alan Austin
*************************************************************************/
/****	Defines */
#ifndef INTERFACE_FAT32
#define INTERFACE_FAT32
/****	Type Definitions */
/****	Prototypes */

/* filesystem types*/
#define FILESYSTEM_UNKNOWN			0x00
#define FILESYSTEM_FAT12			0x01
#define FILESYSTEM_FAT16_MAX32M		0x03
#define FILESYSTEM_EXTMSDOS_NOTLBA	0x05
#define FILESYSTEM_FAT16_NOTLBA		0x06
#define FILESYSTEM_FAT32_NOTLBA		0x0b
#define FILESYSTEM_FAT32			0x0c
#define FILESYSTEM_FAT16			0x0e
#define FILESYSTEM_EXTMSDOS			0x0f

/* drive partition state*/
#define PARTITION_INACTIVE	0x00
#define PARTITION_ACTIVE	0x80


/* boot sector data structure*/

typedef struct
{
									/* Validity Checking Requirements:*/

UBYTE bootcode[446];				/* Not applicable: arbritrary X86 assembler*/

UBYTE partition1_state;				/* Allowed: 0,1. Expected: 1 (partition active).*/
UBYTE partition1_begin_head;		/* Not applicable: LBA addressing used*/
UBYTE partition1_begin_sector;		/* Not applicable: LBA addressing used*/
UBYTE partition1_begin_cylinder;	/* Not applicable: LBA addressing used*/
UBYTE partition1_type;				/* Allowed: type=FILESYSTEM_FAT32*/
UBYTE partition1_end_head;			/* Not applicable: LBA addressing used*/
UBYTE partition1_end_sector;		/* Not applicable: LBA addressing used*/
UBYTE partition1_end_cylinder;		/* Not applicable: LBA addressing used*/
UBYTE partition1_sector_offset[4];	/* Allowed: see next*/
UBYTE partition1_sector_total[4];	/* Allowed: 64MB <= (offset+total)*512 <= 2Gb*/

UBYTE partition2_state;				/* Not applicable: driver only supports single partition*/
UBYTE partition2_begin_head;		/* Not applicable: driver only supports single partition*/
UBYTE partition2_begin_sector;		/* Not applicable: driver only supports single partition*/
UBYTE partition2_begin_cylinder;	/* Not applicable: driver only supports single partition*/
UBYTE partition2_type;				/* Not applicable: driver only supports single partition*/
UBYTE partition2_end_head;			/* Not applicable: driver only supports single partition*/
UBYTE partition2_end_sector;		/* Not applicable: driver only supports single partition*/
UBYTE partition2_end_cylinder;		/* Not applicable: driver only supports single partition*/
UBYTE partition2_sector_offset[4];	/* Not applicable: driver only supports single partition*/
UBYTE partition2_sector_total[4];	/* Not applicable: driver only supports single partition*/

UBYTE partition3_state;				/* Not applicable: driver only supports single partition*/
UBYTE partition3_begin_head;		/* Not applicable: driver only supports single partition*/
UBYTE partition3_begin_sector;		/* Not applicable: driver only supports single partition*/
UBYTE partition3_begin_cylinder;	/* Not applicable: driver only supports single partition*/
UBYTE partition3_type;				/* Not applicable: driver only supports single partition*/
UBYTE partition3_end_head;			/* Not applicable: driver only supports single partition*/
UBYTE partition3_end_sector;		/* Not applicable: driver only supports single partition*/
UBYTE partition3_end_cylinder;		/* Not applicable: driver only supports single partition*/
UBYTE partition3_sector_offset[4];	/* Not applicable: driver only supports single partition*/
UBYTE partition3_sector_total[4];	/* Not applicable: driver only supports single partition*/

UBYTE partition4_state;				/* Not applicable: driver only supports single partition*/
UBYTE partition4_begin_head;		/* Not applicable: driver only supports single partition*/
UBYTE partition4_begin_sector;		/* Not applicable: driver only supports single partition*/
UBYTE partition4_begin_cylinder;	/* Not applicable: driver only supports single partition*/
UBYTE partition4_type;				/* Not applicable: driver only supports single partition*/
UBYTE partition4_end_head;			/* Not applicable: driver only supports single partition*/
UBYTE partition4_end_sector;		/* Not applicable: driver only supports single partition*/
UBYTE partition4_end_cylinder;		/* Not applicable: driver only supports single partition*/
UBYTE partition4_sector_offset[4];	/* Not applicable: driver only supports single partition*/
UBYTE partition4_sector_total[4];	/* Not applicable: driver only supports single partition*/

UBYTE signature[2];					/* Allowed: {0x55, 0xaa}*/

} MBR;

/* Bios Parameter Block Data structure */

typedef struct
{
								/* Validity Checking Requirements:*/

UBYTE jmp_boot[3];				/* Allowed: {0xeb,0x??,0x90} or {0xe9,0x??,0x??}*/
UBYTE oem_name[8];				/* Allowed: arbritrary name string. Expected: "MSWIN*.*"*/
UBYTE bytes_per_sector[2];		/* Allowed: 512,1024,2048,4096. Expected: 512 (Compact Flash)*/
UBYTE sectors_per_cluster;		/* Allowed: 1,2,4,8,16,32,62,128*/
UBYTE reserved_sectors[2];		/* Allowed: non-zero. Expected: 32*/
UBYTE num_fats;					/* Allowed: non-zero. Expected: 2*/
UBYTE root_entries16[2];		/* Allowed: zero (FAT32)*/
UBYTE total_sectors16[2];		/* Allowed: zero (FAT32)*/
UBYTE media_descriptor;			/* Allowed: 0xf0,0xf8-0xff. Expected: 0xf0 (removable) or 0xf8 (fixed)*/
UBYTE fat_size16[2];			/* Allowed: zero (FAT32)*/
UBYTE sectors_per_track[2];		/* Not applicable: CHS addressing only (LBA addressing used)*/
UBYTE num_heads[2];				/* Not applicable: CHS addressing only (LBA addressing used)*/
UBYTE hidden_sectors[4];		/* Allowed: OS specific. Expected: zero (for non-partitioned)*/
UBYTE total_sectors32[4];		/* Allowed: non-zero*/
UBYTE fat_size32[4];			/* Allowed: non-zero*/
UBYTE ext_flags32[2];			/* Not applicable: ill defined*/
UBYTE fs_version32[2];			/* Not applicable: Arbritrary file system major/minor revision number*/
UBYTE root_start_cluster32[4];	/* Allowed: non-zero. Expected: 2*/
UBYTE fsinfo_start_sector32[2];	/* Allowed: non-zero. Expected: 1*/
UBYTE backup_boot_sector[2];	/* Allowed: non-zero. Expected: 6*/
UBYTE reserved[12];				/* Not Applicable*/
UBYTE drive_num;				/* Not Applicable: OS specific*/
UBYTE reserved1;				/* Not Applicable*/
UBYTE boot_signature;			/* Allowed: 0x29*/
UBYTE volume_id[4];				/* Not applicable: user defined value*/
UBYTE volume_label[11];			/* Not applicable: user defined text string*/
UBYTE file_system_type[8];		/* Allowed: "FAT32  "*/

} BPB;

typedef union
{
	UBYTE bb[512];
	UWORD bw[256];
	ULONG bl[128];
	MBR mbr;		/* master boot record */
	BPB bpb;		/* bios parameter block */
} SECTORDATA;



/**************** Directory Record Constants *********************/

/* record field identifiers: alias record */
enum
{
	FILENAME=0,
	EXTENSION=8,
	ATTRIBUTE=11,
	NT_RESERVED=12,
	CRT_TIME_TENTH=13,
	CRT_TIME=14,
	CRT_DATE=16,
	ACCESS_DATE=18,
	START_CLUSTER_HI=20,
	WRT_TIME=22,
	WRT_DATE=24,
	START_CLUSTER_LO=26,
	FILESIZE=28
};

/* record field identifiers: long filename record*/
enum
{
	SEQUENCE_NUM=0,
	NAME1=1,
	NAME2=3,
	NAME3=5,
	NAME4=7,
	NAME5=9,
	ATTRIBUTE_LONG=11,
	RESERVED=12,
	ALIAS_CHECKSUM=13,
	NAME6=14,
	NAME7=16,
	NAME8=18,
	NAME9=20,
	NAME10=22,
	NAME11=24,
	START_CLUSTER=26,
	NAME12=28,
	NAME13=30
};

/* record[0] values: */
#define NULL_ENTRY		0
#define DELETED_ENTRY	0xe5

/* record[ATTRIBUTE] values: */
#define ATTR_READ_ONLY	0x01
#define ATTR_HIDDEN		0x02
#define ATTR_SYSTEM		0x04
#define ATTR_VOLUME_ID	0x08
#define ATTR_DIRECTORY	0x10
#define ATTR_ARCHIVE	0x20
#define ATTR_LONG_NAME	(ATTR_READ_ONLY|ATTR_HIDDEN|ATTR_SYSTEM|ATTR_VOLUME_ID)

/******************************************************************************/

/* FAT32 mount: */

/* FAT32 error codes */
enum {
/* Master Boot Record (MBR) error codes */
MBR_ERROR_NO_PARTITION,
MBR_ERROR_SIGNATURE,
MBR_ERROR_FS_TYPE,
MBR_ERROR_END,
/* Bios Parameter Block (BPB) error codes */
BPB_ERROR_JMP_BOOT,
BPB_ERROR_OEM_NAME,
BPB_ERROR_BYTES_PER_SECTOR,
BPB_ERROR_SECTORS_PER_CLUSTER,
BPB_ERROR_RESERVED_SECTORS,
BPB_ERROR_NUM_FATS,
BPB_ERROR_ROOT_ENTRIES16,
BPB_ERROR_TOTAL_SECTORS16,
BPB_ERROR_MEDIA_DESCRIPTOR,
BPB_ERROR_FAT_SIZE16,
BPB_ERROR_HIDDEN_SECTORS,
BPB_ERROR_TOTAL_SECTORS32,
BPB_ERROR_FAT_SIZE32,
BPB_ERROR_ROOT_START_CLUSTER32,
BPB_ERROR_FSINFO_START_SECTOR32,
BPB_ERROR_BACKUP_BOOT_SECTOR,
BPB_ERROR_BOOT_SIGNATURE,
BPB_ERROR_FILE_SYSTEM_TYPE,
FAT32_ERROR_END
};

/* file write, file seek errors */
enum { FILE_NOT_OPEN=1, FILE_READ_ONLY, FILE_SECTOR_OUT_OF_RANGE};


/* FAT32 error  structure */
typedef struct fat32error {
	UBYTE ident[FAT32_ERROR_END];	/* individual test results*/
	UBYTE result;					/* overall result*/
} FAT32ERROR;


/* FAT32 validation result */
typedef enum fat32status {
	FAT32_OK=0x00,					/* Pass*/
	FAT32_UNEXPECTED=0x01,			/* non-fatal error*/
	FAT32_INVALID=0x02				/* fatal errors*/
} FAT32STATUS;


/* file system descriptor */
typedef struct
{
	UBYTE status;
	UBYTE label[12];
	ULONG partition_size;
	UBYTE sectors_per_cluster;
	ULONG FAT_start_sector;
	ULONG data_start_sector;
	ULONG root_dir_start_cluster;
	ULONG current_dir_start_cluster;
	UBYTE current_path[256];
} FILESYSTEM_DESCRIPTOR;


typedef struct
{
	ULONG next_cluster;
	ULONG next_record;
} DIR_DESCRIPTOR;

typedef struct
{
	UBYTE alias[16];
	UBYTE long_filename[256];
	ULONG filesize;
	UBYTE attrib_str[16];
} DIR_ENTRY;

/*
	Run length encoded cluster list:
	- saves memory space
	- allows multiple sector loading for speed

	Maximum file size depends on degree of fragmentation

	Unfragmented:

  	Max filesize 	= 256*512*2*MAX_CLUSTERS
					= 256*512*2*256
					= 64 Mbytes


	100% fragmented:

  	Max filesize 	= 1*512*2*MAX_CLUSTERS
					= 1*512*2*256
					= 256 Kbytes

	In practice compact flash files are likely to be
  	almost completely unfragmented
*/

#define MAX_CLUSTERS	256

/* Run Length Encoded RAM copy of file cluster list */
typedef struct
{
	ULONG cluster_value[MAX_CLUSTERS];
	/* number of contiguous sectors */
	UBYTE repeat_count[MAX_CLUSTERS];
	/* Current reading position: index */
	UWORD index;
	/* Current reading position: sector offset within repeats */
	UBYTE offset;
} RLE_CLUSTER_LIST;

/* file structure
To support unix style fopen() etc for enhanced driver
See Software Design 80-17203

Static storage for run length encoded cluster list and for
sector buffer (used for character access)

sizeof(FILE_DESCRIPTOR) = 1810 bytes

Number of file descriptors will depend on RAM available

*/
typedef struct
{
	ULONG start_cluster;			/* Start cluster on drive */
	RLE_CLUSTER_LIST cluster_list;	/* Cluster List */
	ULONG current_byte;				/* Current byte position (file relative)  0 <= current_byte < size */
	ULONG current_sector;			/* Current sector (file relative) = current_byte/512 */
	SECTORDATA buffer;				/* Sector Buffer (512 bytes) */
	ULONG size;						/* File size in bytes */
	UBYTE status;					/* Open/Closed */
	UBYTE flags;					/* Future expansion (eg R, RW, contiguous) */
} FILE_DESCRIPTOR;

/* master boot record signature constants */
#define MBR_SIGNATURE_0	((UBYTE)0x55)
#define MBR_SIGNATURE_1	((UBYTE)0xaa)


/* file status */
#define FILE_CLOSED	0x00
#define FILE_OPEN	0x01

/* file mode flags */
#define READ_ONLY		0x01
#define OVERWRITE_ONLY	0x02
#define READ_OVERWRITE	0x03

/* file system status */
#define MOUNTED	1

/* universally true */
#define BYTES_PER_SECTOR	(512)

#define DIR_RECORDS_PER_SECTOR 16

/* FAT entry values */
#define CLUSTER_FREE		0x00
#define CLUSTER_RESERVED	0x0ffffff0
#define CLUSTER_BAD			0x0ffffff7
#define CLUSTER_EOC			0x0ffffff8
/*
	NOTE:
	0x0ffffff8 <= CLUSTER_EOC <= 0x0fffffff

	So test for EOC should be of form:

	if (cluster >= CLUSTER_EOC)		=> end of cluster...

*/

#define EOF	0xfffe

#define CASE_SENSITIVITY	FALSE


/****	Prototypes */

#include	"fat32.p"

#endif

/*************************************************************************/
