/*-------------------------------------------------------------------
Name......... haf.c
Author....... Mark Perkis
Date......... 23/09/03
Description.. Heber animation file format loader.
-------------------------------------------------------------------*/

/*-------------------------------------------------------------------
DECLARATION FORMATS
Type defs........... UPPER CASE
#defines............ UPPER CASE
Global Functions.... GlobalFunction();
Static Functions.... staticfunction();
Global Variables.... GlobalVariable;
Static Variables.... staticvariable;
Local Variables..... localvariable;
Structure Elements.. Structure.element;
-------------------------------------------------------------------*/

/****	Include Files */
#include	"hardware.h"
#include	"interface.h"
#include	"haf.sp"

/****	Externals */

/****	Local Definitions */

/****	Global Variables */

/****	Local Variables */

/****	Global Constants */

/****	Local Constants */

/************************************************************************
*****	Functions
*************************************************************************/
/*-------------------------------------------------------------------
Name......... OpenHAF()
Author....... Mark Perkins
Date......... 23/09/03
Description.. Open Heber Animation File Format.
Parameters... 1) Address of the device
............. 2) File Name
............. 3) Pointer to HAF info structure.
Return....... Returns 0 if successfull
............. Returns code if failed
History...... 26/09/03 - Tidy Up  - MP
............. 18/09/03 - First Pass  - MP
-------------------------------------------------------------------*/
UBYTE	FileOpenHAF(const DEVICE* dev, BYTE *filename, HAFFILEINFO *hafinfo)
{
	HAFFILEINFO *haffileinfo;

	UBYTE	infosector[512];

	UWORD	*infowordmemptr;
	ULONG	*infolongmemptr;

	/* Get pointer to info structure */
	haffileinfo = hafinfo;

	/* Open File */
	if ( FileOpen(dev, filename, &haffileinfo->file, READ_ONLY) != 0 )
	{
		/* Get Pointers to memory */
		(ULONG)infowordmemptr = (ULONG)&infosector[0];
		(ULONG)infolongmemptr = (ULONG)&infosector[0];

		/* Read First Sector as this contains file/image info */
		if ( (FileGetSectors( dev, &haffileinfo->file, infowordmemptr, 1)) == 1 )
		{
			/* Get Height */
			haffileinfo->imageheight = *infolongmemptr++;

			/* Get Width */
			haffileinfo->imagewidth = *infolongmemptr++;

			/* Get Sectors Per Image */
			haffileinfo->sectorsperimage = *infolongmemptr++;

			/* Get Number Of Images */
			haffileinfo->numberofimages = *infolongmemptr++;

			return(0);
		}
		else
		{
			return(1);
		}
	}
	else
	{
		return(FILENOTFOUNDERR);
	}
}

/*-------------------------------------------------------------------
Name......... ReadFrameHAF()
Author....... Mark Perkins
Date......... 23/09/03
Description.. Open Heber Animation File Format.
Parameters... 1) Address of the device
............. 2) Pointer to HAF info structure.
............. 3) memory pointer for data out
Return....... Returns 0 if successfull
............. Returns positive value if failed
History...... 26/09/03 - Tidy Up  - MP
............. 18/09/03 - First Pass  - MP
-------------------------------------------------------------------*/
UBYTE	ReadFrameHAF(const DEVICE* dev, HAFFILEINFO *hafinfo,
					 UWORD *memptr)
{
	HAFFILEINFO *haffileinfo;

	/* Get pointer to info structure */
	haffileinfo = hafinfo;

	/* Get Sectors to memory */
	if ( (FileGetSectors( dev, &haffileinfo->file, memptr,
						  haffileinfo->sectorsperimage)) ==
						  haffileinfo->sectorsperimage )
	{
		return(0);
	}
	else
	{
		return(1);
	}
}

/*-------------------------------------------------------------------
Name......... FileCloseHAF()
Author....... Mark Perkins
Date......... 23/09/03
Description.. Close Heber Animation File.
Parameters... 1) Address of the device
............. 2) Pointer to HAF info structure.
Return....... Returns 0 if successfull
............. Returns positive value if failed
History...... 26/09/03 - Tidy Up  - MP
............. 18/09/03 - First Pass  - MP
-------------------------------------------------------------------*/
void	FileCloseHAF(const DEVICE* dev, HAFFILEINFO *hafinfo)
{
	HAFFILEINFO *haffileinfo;

	/* Get pointer to info structure */
	haffileinfo = hafinfo;

	/* Close File */
	FileClose(&haffileinfo->file );
}


/*-------------------------------------------------------------------
Name......... RewindToFirstFrameHAF()
Author....... Mark Perkins
Date......... 26/09/03
Description.. Rewind to first frame in file.
Parameters... 1) Address of the device
............. 2) Pointer to HAF info structure.
Return....... Returns 0 if successfull
............. Returns positive value if failed
History...... 26/09/03 - First Pass - MP
-------------------------------------------------------------------*/
UBYTE	RewindToFirstFrameHAF(const DEVICE* dev, HAFFILEINFO *hafinfo)
{
	HAFFILEINFO *haffileinfo;

	UBYTE	tempsector[512];

	UWORD	*infowordmemptr;

	/* Get pointer to info structure */
	haffileinfo = hafinfo;

	/* Get pointer to temp memory area */
	(ULONG)infowordmemptr = (ULONG)&tempsector[0];

	/* Rewind to start of file and read first sector to get to first image */
	FileRewind(&haffileinfo->file);

	if ( (FileGetSectors( dev, &haffileinfo->file, infowordmemptr, 1)) == 1 )
	{
		return(0);
	}
	else
	{
		return(1);
	}
}